//
//  PlayerViewController.swift
//  AMPTVOSSample
//
//  Created by Cortes Saenz, David on 10/23/15.
//  Copyright © 2015 Akamai. All rights reserved.
//

import UIKit
import AVKit

import AmpCoreTV
import AmpMediaAnalyticsTV
import AmpOctoshapeTV
import AmpStatisticsTV

public class PlayerViewController: UIViewController, AVPlayerViewControllerDelegate, PlayerEventObserver, AmpOctoshapeObserver {
    
    public var ampPlayer: AmpPlayer!
    public var playerController: AVPlayerViewController!
    public var ampMA: AmpMediaAnalyticsManager!
    public var ampOctoshape: AmpOctoshapeManager!
    public var ampStatistics: AmpStatisticsManager!
    var time: Double!;
    var playerId: Int!
    
    let octoSourceTitle = "Octoshape Source:"
    let maBeaconTitle = "Media Analytics Beacon:"
    
    let imaPreMidPostRoll = "http://pubads.g.doubleclick.net/gampad/ads?sz=640x480&iu=/124319096/external/ad_rule_samples&ciu_szs=300x250&ad_rule=1&impl=s&gdfp_req=1&env=vp&output=vmap&unviewed_position_start=1&cust_params=deployment%3Ddevsite%26sample_ar%3Dpremidpost&cmsid=496&vid=short_onecue&correlator="
    
    override public func viewDidLoad() {
        // super.viewDidLoad()
//        NSNotificationCenter.defaultCenter().addObserver(self, selector: Selector("appGoesToBackground:"), name:UIApplicationWillResignActiveNotification, object: nil)
//        NSNotificationCenter.defaultCenter().addObserver(self, selector: Selector("appCameBackFromBackground:"), name:UIApplicationDidBecomeActiveNotification, object: nil)
    }
    
    override public func pressesEnded(presses: Set<UIPress>, withEvent event: UIPressesEvent?) {
        for item in presses {
            if item.type == .PlayPause {
                print("PlayPause")
            }
        }
    }
    
//    func appGoesToBackground(notification : NSNotification) {
//        if let player = self.ampPlayer {
//            player.pause();
//            time = player.currentTime()
//        }
//    }
    
    func appCameBackFromBackground(notification : NSNotification) {
        self.ampPlayer.seekTo(self.time)
    }
    
    public func playerViewController(playerViewController: AVPlayerViewController, willResumePlaybackAfterUserNavigatedFromTime oldTime: CMTime, toTime targetTime: CMTime) {
        self.ampMA.notifyEndScrub()
    }
    
    public func playVideo(videoUrl: String, playerController: AVPlayerViewController) {
        
        self.ampPlayer = AmpPlayer()
        
        print("Amp Version: \(self.ampPlayer.version)")
        
        self.setupPlayerController(playerController)
        
        // Register
        playerId = self.ampPlayer.registerObserver(self)
        
        self.ampOctoshape = AmpOctoshapeManager(ampPlayer: self.ampPlayer)
        self.ampOctoshape.registerObserver(self)
        
        self.ampStatistics = AmpStatisticsManager(ampPlayer: self.ampPlayer, parentView: self.playerController.view, containerFrame: self.playerController.view.frame)
        
        self.ampPlayer.handleUrl(videoUrl)
        
        let times = [2.0, 4.0, 6.0, 8.0]
        _ = CuepointManager(ampPlayer: self.ampPlayer, observer: self, times: times)
        
        self.setupAnalytics()
        
        self.ampStatistics.addSegment("Media Analytics Beacon:", info: self.ampMA.configUrl)
        
    }
    
    func setupAnalytics() {
        let analyticsUrl = "https://ma1019-r.analytics.edgekey.net/config/beacon-8566.xml"
        
        let customData = [
            "title": "AKaMTitle",
            "category": "AKamCategory",
            "playerId": "AKAM AVPlayer playerId",
            "device": "AKAM AVPlayer device",
            "show": "AKAM AVPlayer$show"
        ]
        
        self.ampMA = AmpMediaAnalyticsManager(ampPlayer: self.ampPlayer!, configUrl: analyticsUrl, customData: customData)
    }
    
    func setupPlayerController(playerController: AVPlayerViewController) {
        self.playerController = playerController
        
        self.playerController.delegate = self
        self.playerController.player = self.ampPlayer.player
        self.playerController.view.frame = self.view.frame
        self.addChildViewController(self.playerController)
        self.view.addSubview(self.playerController.view)
    }
    
    override public func didReceiveMemoryWarning() {
        // super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    public func onPlaybackStateChanged(ampPlayer: AmpPlayer) {
        //        print("onPlaybackStateChanged")
        if (ampPlayer.playbackState == PlaybackState.Failed) {
            // Handle error
        }
    }
    
    public func onBufferingStateChanged(ampPlayer: AmpPlayer) {
        //        print("onBufferingStateChanged")
        if (ampPlayer.bufferingState == BufferingState.Ready) {
            
            self.ampPlayer.play()
            
            print("BufferingState.Ready")
        }
    }
    
    public func onError(ampPlayer: AmpPlayer, error: PlayerError) {
        switch(error) {
        case PlayerError.UnknownAssetFormat(let assetUrl):
            print("PlayerError.UnknownAssetFormat: ", assetUrl)
        case PlayerError.DownloadFailed():
            print("PlayerError.DownloadFailed")
        case PlayerError.UnknownError(let message):
            print("PlayerError.UnknownError: ", message)
        }
    }
    
    public func onPositionChange(ampPlayer: AmpPlayer) {
        //        print("time: %d", ampPlayer.currentTime())
    }
    
    public func onCuepointReached(ampPlayer: AmpPlayer, atTime: Double) {
        print("cuepoint: %d - time: %d", atTime, ampPlayer.currentTime())
    }
    
    public func onCurrentStreamInfoEvent(currentStream: OCCurrentStreamInfoEvent) {
        print("CurrentStreamInfo - bitrate: %d", currentStream.rateset)
        self.ampStatistics.updateSegment(AmpStatisticsManager.bitrateTitle, info: AmpStatisticsManager.formatBitrate(Double(currentStream.rateset)))
        
    }
    
    public func onStreamInformationChanged(ampPlayer: AmpPlayer, streamInfo: StreamInfo) {
        
    }
    
    public func onMultiStreamInfoEvent(multiStream: OCMultiStreamInfoEvent) {
        multiStream.baseStreams.map({stream in stream as! NSDictionary})
            .forEach { print("Stream:%@ - bitrate: %@, channel: %@", String($0), $0["bitrate"], $0["name"]) }
    }
    
    public func onStreamSourceInfoEvent(streamSource: OCStreamSourceInfoEvent) {
        if let streamSource = streamSource.streamSource {
            print("Source: ", streamSource)
            if self.ampStatistics.hasSegment(octoSourceTitle) {
                self.ampStatistics.updateSegment(octoSourceTitle, info: streamSource)
            } else {
                self.ampStatistics.addSegment(octoSourceTitle, info: streamSource)
            }
        }
        
        if let multiCastStatus = streamSource.multiCastStatus {
            print("MultiCast Status: ", multiCastStatus)
        }
    }
    
    // When we go back to collection view
    override public func viewWillDisappear(animated:Bool) {
        self.ampPlayer.stop()
        self.ampPlayer = nil
        print("Stopped")
        self.playerController.player = nil
        super.viewWillDisappear(animated)
    }
    
}

